#include <assert.h>
#include <unistd.h>
#include <stdarg.h>

#include "rally.h"

int move_fd = -1;

int number_of_players;
int board[MAX_BOARD_SIZE][MAX_BOARD_SIZE];

void send_printf(int fd, char *s, ...) {
	va_list va;
	static char buf[1024];
	int len;

	va_start(va, s);
	len = vsnprintf(buf, 1024, s, va);
	va_end(va);

	if (write(fd, buf, len) == -1 && errno != EPIPE) {
		if (errno == ECONNRESET) {
			/* do something? */
			return;
		} else
			bailsys("write() failed");
	}
}

int valid_moves(char *moves) {
	int i, repair = 0;
	/* validate moves */
	for (i = 0; i < MOVES_PER_ROUND; i++) {
		switch (moves[i]) {
			case MOVE_REPAIR:
				repair = 1;
			case MOVE_NORTH:
			case MOVE_SOUTH:
			case MOVE_EAST:
			case MOVE_WEST:
			case MOVE_TURN_LEFT:
			case MOVE_TURN_RIGHT:
			case MOVE_SHOOT:
			case MOVE_PUNCH:
			case MOVE_NOTHING:
				break;
			default:
				return 0;
		}
	}
	if (repair)
		for (i = 0; i < MOVES_PER_ROUND; i++)
			if (moves[i] != MOVE_REPAIR)
				return 0;
	return 1;
}

void move_player(int player_n, move m, int move_num, int should_shoot) {
	/* should_shoot = 0 if not at all, 1 if only shoot, 2 if don't care */
	player *p = players+player_n;
	int sr = 0, sc = 0, fr = -1, fc = -1, r, c, hit;
	debug("Moving player %d with move %c, on move number %d", player_n, m, move_num);
	if (should_shoot > 0) {
		switch(m) {
			case MOVE_SHOOT:
				switch (players[player_n].dir) {
					case NORTH:
						sr = 0;
						fr = players[player_n].r-1;
						sc = players[player_n].c;
						fc = players[player_n].c;
						break;
					case SOUTH:
						sr = players[player_n].r+1;
						fr = board_size-1;
						sc = players[player_n].c;
						fc = players[player_n].c;
						break;
					case EAST:
						sr = players[player_n].r;
						fr = players[player_n].r;
						sc = players[player_n].c+1;
						fc = board_size-1;
						break;
					case WEST:
						sr = players[player_n].r;
						fr = players[player_n].r;
						sc = 0;
						fc = players[player_n].c-1;
						break;
				}
				hit = 0;
				for (r = sr; r <= fr && !hit; r++) {
					if (r < 0 || r >= board_size)
						continue;
					for (c = sc; c <= fc && !hit; c++) {
						if (c < 0 || c >= board_size)
							continue;
						if (board[r][c] != -1) {
							slog("Fire from %d hit %d", player_n, board[r][c]);
							if (players[board[r][c]].health > 0)
								players[board[r][c]].health--;
							if (players[board[r][c]].health <= 0) {
#ifdef SERVER
								broadcast_printf("DEAD %d\n", board[r][c]);
								clients[board[r][c]].state = STATE_DEAD;
								players_alive--;
								if (players_alive == 1)
									winner = player_n;
#endif
								board[r][c] = -1;
							}
							hit = 1;
						}
					}
				}
				break;
			case MOVE_PUNCH:
				switch (players[player_n].dir) {
					case NORTH:
						sr = players[player_n].r-1;
						fr = players[player_n].r-1;
						sc = players[player_n].c-1;
						fc = players[player_n].c+1;
						break;
					case SOUTH:
						sr = players[player_n].r+1;
						fr = players[player_n].r+1;
						sc = players[player_n].c-1;
						fc = players[player_n].c+1;
						break;
					case EAST:
						sr = players[player_n].r-1;
						fr = players[player_n].r+1;
						sc = players[player_n].c+1;
						fc = players[player_n].c+1;
						break;
					case WEST:
						sr = players[player_n].r-1;
						fr = players[player_n].r+1;
						sc = players[player_n].c-1;
						fc = players[player_n].c-1;
						break;
				}
				for (r = sr; r <= fr; r++) {
					if (r < 0 || r >= board_size)
						continue;
					for (c = sc; c <= fc; c++) {
						if (c < 0 || c >= board_size)
							continue;
						if (board[r][c] != -1) {
							players[board[r][c]].health--;
							if (players[board[r][c]].health <= 0) {
#ifdef SERVER
								broadcast_printf("DEAD %d\n", board[r][c]);
								clients[board[r][c]].state = STATE_DEAD;
								players_alive--;
								if (players_alive == 1)
									winner = player_n;
#endif
								board[r][c] = -1;
							}
						}
					}
				}
				break;
			default:
				if (should_shoot == 1)
					return;
		}
	}
	switch(m) {
		case MOVE_NORTH:
			if (p->r > 0 && board[p->r-1][p->c] == -1) {
				board[p->r][p->c] = -1;
				p->r--;
				board[p->r][p->c] = player_n;
			} else {
				slog("E: Player %d made an invalid move (%c from %d,%d)", player_n, m, p->r, p->c);
			}
			break;
		case MOVE_SOUTH:
			if (p->r < (board_size-1) && board[p->r+1][p->c] == -1) {
				board[p->r][p->c] = -1;
				p->r++;
				board[p->r][p->c] = player_n;
			} else {
				slog("E: Player %d made an invalid move (%c)", player_n, m);
			}
			break;
		case MOVE_WEST:
			if (p->c > 0 && board[p->r][p->c-1] == -1) {
				board[p->r][p->c] = -1;
				p->c--;
				board[p->r][p->c] = player_n;
			} else {
				slog("E: Player %d made an invalid move (%c)", player_n, m);
			}
			break;
		case MOVE_EAST:
			if (p->c < (board_size-1) && board[p->r][p->c+1] == -1) {
				board[p->r][p->c] = -1;
				p->c++;
				board[p->r][p->c] = player_n;
			} else {
				slog("E: Player %d made an invalid move (%c)", player_n, m);
			}
			break;
		case MOVE_TURN_LEFT:
			p->dir = (p->dir+3)%4;
			break;
		case MOVE_TURN_RIGHT:
			p->dir = (p->dir+1)%4;
			break;
		case MOVE_REPAIR:
			if (move_num == MOVES_PER_ROUND && p->health != 0 && p->health < STARTING_HEALTH)
				p->health++;
			break;
		default:
			return;
	}
}
