#ifndef _RALLY_H_
#define _RALLY_H_

#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <signal.h>
#include <string.h>

#include "player.h"

#define bailsys(s, x...) { \
    fprintf(stderr, s": %s\n", ## x, strerror(errno)); \
	raise(SIGABRT); \
}

#define bail(s, x...) { \
    fprintf(stderr, s"\n", ## x); \
	raise(SIGABRT); \
}

#define broadcast_printf(s, x...) { \
	int local_i; \
	for (local_i = 0; local_i < MAX_CLIENTS; local_i++) \
		if (clients[local_i].fd != -1) \
			send_printf(clients[local_i].fd, s, ## x); \
	}

enum {
	STATE_WATCHER = -1,
	STATE_UNINIT = 0,
	STATE_READY,
	STATE_PLAYING,
	STATE_DEAD,
};

#ifdef SERVER
#define MSG_LEN 40

typedef struct {
	int fd;
	int state;
	char msg[MSG_LEN+1];
	char *msgp;
	char nextmove[MOVES_PER_ROUND];
} client;

extern client *clients;
#endif

#define RALLY_DEFAULT_HOST	"localhost"
#define RALLY_DEFAULT_PORT	65456

#define log(s, x...) fprintf(stderr, s"\n", ##x)
#ifdef DEBUG

#define debug(s, x...) fprintf(stderr, s"\n", ##x)
#ifdef SERVER
#define slog(s, x...) fprintf(stderr, s"\n", ##x)
#endif

#else

#define debug(x...) do {} while (0)
#define slog(x...) do {} while (0)

#endif

int rally_connect();
int rally_disconnect();
void rally_wait_for_game();
int valid_moves(char *moves);
void move_player(int player_n, move m, int move_num, int should_shoot);
void send_printf(int fd, char *s, ...);
void set_name(char* name);
void wait_until_ready_to_move();
void submit_moves(char *moves);
void wait_for_results(void (*printfunc)());
void am_watcher();

extern int number_of_players, players_alive, winner;
extern int board[MAX_BOARD_SIZE][MAX_BOARD_SIZE];
extern int board_size;
extern player *players;

#endif /* _RALLY_H_ */
